@@ -30,9 +30,13 @@
 
 #include <stdio.h>
 #include <locale.h>
+
+#ifdef HAVE__NL_TIME_FIRST_WEEKDAY
 #include <langinfo.h>
+#endif
 
 #include <glib.h>
+#include <glib/gstdio.h>
 #include <glib/gprintf.h>
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
@@ -129,6 +133,7 @@ GtkWidget *ical_weekstartday_combobox;
  *     to return 0..6 mon..sun, which is what libical uses */
 int get_first_weekday_from_locale()
 {
+#ifdef HAVE__NL_TIME_FIRST_WEEKDAY
     union { unsigned int word; char *string; } langinfo;
     int week_1stday = 0;
     int first_weekday = 1;
@@ -147,6 +152,10 @@ int get_first_weekday_from_locale()
         orage_message(150, "get_first_weekday: unknown value of _NL_TIME_WEEK_1STDAY.");
 
     return((week_1stday + first_weekday - 2 + 7) % 7);
+#else
+    orage_message(150, "get_first_weekday: Can not find first weekday. Using default: Monday=0. If this is wrong guess. please set undocumented parameter: Ical week start day (Sunday=6)");
+    return(0);
+#endif
 }
 
 static void dialog_response(GtkWidget *dialog, gint response_id
@@ -314,6 +323,17 @@ static void pager_changed(GtkWidget *dialog, gpointer user_data)
 
 static void set_systray()
 {
+#if GTK_CHECK_VERSION(2,10,0)
+    if (!(g_par.trayIcon 
+    && gtk_status_icon_is_embedded((GtkStatusIcon *)g_par.trayIcon))) {
+        g_par.trayIcon = create_TrayIcon();
+    }
+
+    if (g_par.show_systray)
+        gtk_status_icon_set_visible((GtkStatusIcon *)g_par.trayIcon, TRUE);
+    else
+        gtk_status_icon_set_visible((GtkStatusIcon *)g_par.trayIcon, FALSE);
+#else
     if (!(g_par.trayIcon 
     && NETK_IS_TRAY_ICON(((XfceTrayIcon *)g_par.trayIcon)->tray))) {
         g_par.trayIcon = create_TrayIcon();
@@ -323,6 +343,7 @@ static void set_systray()
         xfce_tray_icon_connect((XfceTrayIcon *)g_par.trayIcon);
     else
         xfce_tray_icon_disconnect((XfceTrayIcon *)g_par.trayIcon);
+#endif
 }
 
 static void systray_changed(GtkWidget *dialog, gpointer user_data)
@@ -390,13 +411,13 @@ static void timezone_button_clicked(GtkButton *button, gpointer user_data)
 {
     Itf *itf = (Itf *)user_data;
 
-    if (g_par.local_timezone == NULL || strlen(g_par.local_timezone) == 0) {
+    if (!ORAGE_STR_EXISTS(g_par.local_timezone)) {
         g_warning("timezone pressed: local timezone missing");
         g_par.local_timezone = g_strdup("floating");
     }
     if (xfical_timezone_button_clicked(button, GTK_WINDOW(itf->orage_dialog)
             , &g_par.local_timezone))
-        xfical_set_local_timezone();
+        xfical_set_local_timezone(FALSE);
 }
 
 #ifdef HAVE_ARCHIVE
@@ -682,8 +703,9 @@ static void create_parameter_dialog_display_tab(Itf *dialog)
 
 static void create_parameter_dialog_extra_setup_tab(Itf *dialog)
 {
-    GtkWidget *hbox, *vbox, *label, *event;
+    GtkWidget *hbox, *vbox, *label;
     /* code removed. relying in get_first_weekday_from_locale now
+    GtkWidget *event;
     gchar *weekday_array[7] = {
             _("Monday"), _("Tuesday"), _("Wednesday"), _("Thursday")
           , _("Friday"), _("Saturday"), _("Sunday")};
@@ -938,6 +960,88 @@ void write_parameters()
     orage_rc_file_close(orc);
 }
 
+/* let's try to find the timezone name by comparing this file to
+ * timezone files from the default location. This does not work
+ * always, but is the best trial we can do */
+void init_dtz_check_dir(gchar *tz_dirname, gchar *tz_local, gint len)
+{
+    gint tz_offset = strlen("/usr/share/zoneinfo/");
+    gsize tz_len;         /* file lengths */
+    GDir *tz_dir;         /* location of timezone data /usr/share/zoneinfo */
+    const gchar *tz_file; /* filename containing timezone data */
+    gchar *tz_fullfile;   /* full filename */
+    gchar *tz_data;       /* contents of the timezone data file */
+    GError *error = NULL;
+
+    if ((tz_dir = g_dir_open(tz_dirname, 0, NULL))) {
+        while ((tz_file = g_dir_read_name(tz_dir)) 
+                && g_par.local_timezone == NULL) {
+            tz_fullfile = g_strconcat(tz_dirname, "/", tz_file, NULL);
+            if (g_file_test(tz_fullfile, G_FILE_TEST_IS_SYMLINK))
+                ; /* we do not accept these, just continue searching */
+            else if (g_file_test(tz_fullfile, G_FILE_TEST_IS_DIR)) {
+                init_dtz_check_dir(tz_fullfile, tz_local, len);
+            }
+            else if (g_file_get_contents(tz_fullfile, &tz_data, &tz_len
+                    , &error)) {
+                if (len == tz_len && !memcmp(tz_local, tz_data, len)) {
+                    /* this is a match (length is tested first since that 
+                     * test is quick and it eliminates most) */
+                    g_par.local_timezone = g_strdup(tz_fullfile+tz_offset);
+                }
+                g_free(tz_data);
+            }
+            else { /* we should never come here */
+                g_warning("init_default_timezone: can not read (%s) %s"
+                        , tz_fullfile, error->message);
+                g_error_free(error);
+                error = NULL;
+            }
+            /* if we found a candidate, test that libical knows it */
+            if (g_par.local_timezone && !xfical_set_local_timezone(TRUE)) { 
+                /* candidate is not known by libical, remove it */
+                g_free(g_par.local_timezone);
+                g_par.local_timezone = NULL;
+            }
+            g_free(tz_fullfile);
+        } /* while */
+        g_dir_close(tz_dir);
+    }
+}
+
+void init_default_timezone()
+{
+    gsize len;            /* file lengths */
+    gchar *tz_local;      /* local timezone data */
+
+    g_free(g_par.local_timezone);
+    g_par.local_timezone = NULL;
+    g_message(_("First Orage start. Searching default timezone."));
+    /* debian, ubuntu stores the timezone name into /etc/timezone */
+    if (g_file_get_contents("/etc/timezone", &g_par.local_timezone
+                , &len, NULL)) {
+        /* success! let's check it */
+        if (len > 2) /* get rid of the \n at the end */
+            g_par.local_timezone[len-1] = 0;
+            /* we have a candidate, test that libical knows it */
+        if (!xfical_set_local_timezone(TRUE)) { 
+            g_free(g_par.local_timezone);
+            g_par.local_timezone = NULL;
+        }
+    }
+    /* redhat, gentoo, etc. stores the timezone data into /etc/localtime */
+    else if (g_file_get_contents("/etc/localtime", &tz_local, &len, NULL)) {
+        init_dtz_check_dir("/usr/share/zoneinfo", tz_local, len);
+        g_free(tz_local);
+    }
+    if (ORAGE_STR_EXISTS(g_par.local_timezone))
+        g_message(_("Default timezone set to %s."), g_par.local_timezone);
+    else {
+        g_par.local_timezone = g_strdup("floating");
+        g_message(_("Default timezone not found, please, set it manually."));
+    }
+}
+
 void read_parameters(void)
 {
     gchar *fpath;
@@ -947,7 +1051,10 @@ void read_parameters(void)
 
     orc = orage_parameters_file_open(TRUE);
 
-    g_par.local_timezone = orage_rc_get_str(orc, "Timezone", "floating");
+    g_par.local_timezone = orage_rc_get_str(orc, "Timezone", "not found");
+    if (!strcmp(g_par.local_timezone, "not found")) { 
+        init_default_timezone();
+    }
 #ifdef HAVE_ARCHIVE
     g_par.archive_limit = orage_rc_get_int(orc, "Archive limit", 0);
     fpath = orage_data_file_location(ORAGE_ARC_FILE);
@@ -1017,5 +1124,5 @@ void set_parameters()
     */
     set_stick();
     set_ontop();
-    xfical_set_local_timezone();
+    xfical_set_local_timezone(FALSE);
 }
